@@ -82,6 +82,9 @@ def log_exp_p_1_fn(x: torch.Tensor):
 generalfunctions = {
     "gelu": torch.nn.functional.gelu,
     "relu": torch.relu,
+    "step": lambda x: torch.where(
+        x > 0.0, torch.ones((), dtype=x.dtype, device=x.device), torch.zeros((), dtype=x.dtype, device=x.device)
+    ),
     "sigmoid": torch.sigmoid,
     "rsqrt": torch.rsqrt,
     "reciprocal": torch.reciprocal,
