@@ -1,23 +1,23 @@
-use crate::{all_imports::Shape, hashmaps::FxHashMap as HashMap};
+use crate::{all_imports::Shape, circuit::repr::RustRepr, hashmaps::FxHashMap as HashMap};
 
 use num_bigint::BigUint;
 
 use crate::{circuit::print::rust_expression_notation_circuit, pyo3_prelude::*};
 
 use super::{
-    algebraic_rewrite::deep_optimize_einsums,
     canonicalize::deep_canonicalize,
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::strip_names,
     debugging::{create_circuit_test, randn_ablate_great_grandchildren},
     deep_rewrite::deep_heuristic_nest_adds,
-    deep_rewrite::{compiler_simp, deep_push_down_index},
+    deep_rewrite::{compiler_simp, deep_optimize_einsums, deep_push_down_index},
     flat_concat, get_compatible_dtype,
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, Schedule, SchedulingError,
     },
     CircuitNode, CircuitRc, HashBytes,
 };
+use crate::opt_einsum::EinsumSpec;
 use crate::{py_types::Tensor, timed, util::apply_fn_until_same};
 
 // a lot of the function boundries in this are there because it would be too hard to optimize things jointly
@@ -54,6 +54,8 @@ pub struct OptimizationSettings {
     pub create_tests: bool,
     #[pyo3(get, set)]
     pub log_simplifications: bool,
+    #[pyo3(get, set)]
+    pub log_slow_einsums: bool,
 }
 
 #[cfg(not(feature = "real-pyo3"))]
@@ -79,6 +81,7 @@ impl Default for OptimizationSettings {
             capture_and_print: false,
             create_tests: false,
             log_simplifications: false,
+            log_slow_einsums: false,
         }
     }
 }
@@ -99,7 +102,8 @@ impl OptimizationSettings {
         numerical_scale_max = "OptimizationSettings::default().numerical_scale_max",
         capture_and_print = "OptimizationSettings::default().capture_and_print",
         create_tests = "OptimizationSettings::default().create_tests",
-        log_simplifications = "OptimizationSettings::default().log_simplifications"
+        log_simplifications = "OptimizationSettings::default().log_simplifications",
+        log_slow_einsums = "OptimizationSettings::default().log_slow_einsums"
     )]
     fn new(
         verbose: usize,
@@ -115,6 +119,7 @@ impl OptimizationSettings {
         capture_and_print: bool,
         create_tests: bool,
         log_simplifications: bool,
+        log_slow_einsums: bool,
     ) -> Self {
         Self {
             verbose,
@@ -130,6 +135,7 @@ impl OptimizationSettings {
             capture_and_print,
             create_tests,
             log_simplifications,
+            log_slow_einsums,
         }
     }
 }
@@ -145,6 +151,7 @@ pub struct OptimizationCache {
     pub simplification_log: Vec<&'static str>,
     pub max_tensor_elements: usize,
     pub max_single_tensor_numel: usize,
+    pub slow_einsum_log: Vec<EinsumSpec>,
 }
 
 #[derive(Debug, Clone, Default)]
@@ -233,6 +240,9 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
         println!("Final Circuit");
         circuit.compiler_print();
     }
+    if context.settings.log_slow_einsums {
+        println!("{}", context.cache.slow_einsum_log.repr());
+    }
     circuit
 }
 
