@@ -74,7 +74,7 @@ pub fn assert_is_close(a: CircuitRc, b: CircuitRc) {
 #[pyfunction]
 pub fn opt_eval_each_subcircuit_until_fail(circuit: CircuitRc, settings: OptimizationSettings) {
     let max_numel = BigUint::from(2_000_000_000usize);
-    let topo = toposort_circuit(circuit);
+    let topo = toposort_circuit(circuit.clone());
     for circ in topo {
         if circ.info().numel() < max_numel {
             let reference = evaluate_fn(&circ).unwrap();
@@ -86,10 +86,7 @@ pub fn opt_eval_each_subcircuit_until_fail(circuit: CircuitRc, settings: Optimiz
                     circ.compiler_print();
                     optimize_circuit(
                         circ,
-                        &mut OptimizationContext {
-                            settings,
-                            cache: Default::default(),
-                        },
+                        &mut OptimizationContext::new_settings_circuit(settings, circuit.clone()),
                     )
                     .compiler_print();
                 }
