@@ -1,10 +1,10 @@
 use crate::circuit::scatter_rewrite::concat_to_scatter;
 use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 
-use crate::pyo3_prelude::*;
-use num_bigint::BigUint;
-
+use crate::circuit::deep_map_op_context;
 use crate::util::{apply_fn_until_same, mapping_until_end, AsOp};
+use crate::{pyo3_prelude::*, timed_value};
+use num_bigint::BigUint;
 
 use super::{
     algebraic_rewrite::*,
@@ -348,3 +348,29 @@ pub fn deep_pull_concat(circuit: CircuitRc, min_size: Option<usize>) -> CircuitR
         result
     })
 }
+
+#[pyfunction]
+#[pyo3(name = "deep_optimize_einsums")]
+pub fn deep_optimize_einsums_py(circ: CircuitRc) -> CircuitRc {
+    deep_optimize_einsums(circ, &mut Default::default())
+}
+
+pub fn deep_optimize_einsums(circ: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
+    deep_map_op_context(
+        &**circ,
+        &|x: &Circuit, context: &mut OptimizationContext| match x {
+            Circuit::Einsum(ein) => {
+                let (result, took) = timed_value!(einsum_nest_optimize(ein, context));
+                let result = result?;
+                if took.as_millis() > 10 {
+                    context.cache.slow_einsum_log.push(ein.get_spec());
+                }
+                Some(result.rc())
+            }
+            _ => None,
+        },
+        context,
+        &mut HashMap::new(),
+    )
+    .unwrap_or(circ.clone())
+}
