@@ -39,7 +39,7 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
             env!("CARGO_MANIFEST_DIR"),
             "/rust_circuit_type_utils.py"
         )),
-        "rust_circuit_type_utils.py",
+        concat!(env!("CARGO_MANIFEST_DIR"), "/rust_circuit_type_utils.py"),
         "rust_circuit_type_utils",
     )
     .unwrap();
