@@ -452,13 +452,11 @@ impl TensorIndex {
                     check(at, start, l, true)?;
                     check(at, stop, l, true)?;
 
-                    if start > stop {
-                        // start and stop have to be Some(x) to get here
-                        let start_u = start.unwrap();
-                        let stop_u = stop.unwrap();
-                        if stop_u < 0 && start_u > stop_u + (l as i64) {
-                            get_err(at)?;
-                        }
+                    let mod_l_idx = |x| if x < 0 { l as i64 + x } else { x };
+                    let start_u: i64 = start.map_or(0, mod_l_idx);
+                    let stop_u: i64 = stop.map_or(l as i64, mod_l_idx);
+                    if start_u > stop_u {
+                        get_err(at)?;
                     }
                 }
                 _ => (),
